# Standard Ethernet

Ethernet è uno standard che definisce le regole per l'invio e la ricezione dei dati tra dispositivi all'interno di una rete LAN (Local Area Network). Esso opera a due livelli del modello OSI:

1. **Livello 1 – Fisico (Physical Layer)**
   - Specifica le caratteristiche fisiche dei componenti, come cavi, connettori, interfacce, moduli di trasmissione, ecc.
   - Nel contesto delle reti cablate (*wired*) la norma IEEE 802.3 definisce tutti gli aspetti relativi alla trasmissione dei segnali su cavi (reti cablate).
   - Nel contesto delle reti *wireless* la norma IEEE 802.11 si occupa delle caratteristiche della trasmissione senza fili, garantendo la compatibilità e le modalità comunicative delle reti wireless.

2. **Livello 2 – Data Link Layer**
   - Definisce la struttura dei pacchetti (*frame*) che viaggiano in rete.
   - Stabilisce i metodi di indirizzamento attraverso l’uso dei MAC-address

## Cavi fisici

Esistono vari tipi di cavi, caratterizzati da differenti lunghezze e capacità di trasmissione, che dipendono dalla tecnologia impiegata.

Possiamo classificarli in base alla velocità supportata:

- Ethernet: fino a 10 Mbps
- Fast Ethernet: fino a 100 Mbps
- Gigabit Ethernet: fino a 1000 Mbps (1 Gbps)
- 10 Gigabit Ethernet: fino a 10 Gbps

Inoltre, i cavi si distinguono anche per la loro composizione: possono essere realizzati in rame oppure in fibra ottica.

![cavo-ethernet](/asset/img/cavo-ethernet.png)

Il cavo in figura è un cavo `UTP` (*Unshielded Twisted Pair*), ovvero non schermato, con coppie di cavi intrecciati. Questa particolare struttura serve a ridurre le interferenze elettromagnetiche e a prevenire il fenomeno del *crosstalk*, cioè l'interferenza tra i segnali che viaggiano su cavi adiacenti. Alle estremità, il cavo è dotato di connettori; il più comune è l'`RJ-45`.

Il cavo UTP è composto da 8 fili di rame, progettati per collegarsi ai pin della porta in cui viene inserito il connettore.

![rj45](/asset/img/rj45.png)

I pin di contatto dei dispositivi devono avere funzionalità opposte affinché la comunicazione possa avvenire correttamente. Ad esempio, i pin 1 e 2 di un dispositivo A devono inviare, mentre i pin 1 e 2 del dispositivo B devono ricevere, e viceversa per gli altri pin.

### Cable pinout Ethernet e Fast Ethernet

Vengono utilizzati solo 4 pin. Per quanto riguarda le connessioni Ethernet e Fast Ethernet, esistono due tipologie di pinout:

- `Straight Through Pinout` (cavo dritto): in questo tipo di cablaggio i cavi vengono collegati in modo diretto: il pin 1 di A va al pin 1 di B, il 2 di A al 2 di B, e così via. I dispositivi hanno funzionalità opposte sui rispettivi pin (trasmettitore e ricevitore), per cui la comunicazione avviene correttamente.

![straight-through-pinout](/asset/img/straight-through-pinout.png)

Un cavo con all'estremità un straight-through cable pinout é utile per mettere in comunicazione due dispositivi che, sugli stessi pin di contatto, hanno funzionalità opposte (sul pin 1 l'host A invia e sul pin 1 l'host B riceve).

- `Crossover Pinout`: in questo schema la disposizione dei pin alle estremità è *incrociata* (1A-3B, 2A-6B, 3A-1B, 6A-2B), permettendo così una comunicazione bidirezionale tra i dispositivi.

Il crossover pinout serve per collegare due dispositivi che, sugli stessi pin di contatto, assegnano la medesima funzionalità. Senza l'incrocio dei cavi non è quindi possibile instaurare una comunicazione bidirezionale.

![crossover-pinout](/asset/img/crossover-pinout.png)

Di solito, dispositivi identici (ad esempio, switch collegati tra loro) assegnano la medesima funzionalità agli stessi pin. Pertanto, per stabilire una comunicazione diretta tra di essi, viene utilizzato un cavo con pinout crossover. Nel caso di dispositivi differenti, come ad esempio un computer e uno switch, le funzionalità dei pin non coincidono: il computer trasmette su un set di pin mentre lo switch riceve su quelli corrispondenti. Pertanto, per collegare dispositivi differenti si utilizza tipicamente un cavo con pinout straight-through.

![device-connection](/asset/img/device-connection.png)

### Cable pinout Gigabit Ethernet

Vengono utilizzati tutti gli 8 pin, con le medesime tipologie di pinout. I pin lavorano sempre a coppie: ad esempio, i ping 1-2A inviano e i pin 1-2B ricevono, ecc. (nel caso del cablaggio straight-through)
